require "import"
import "android.widget.*"
import "com.androlua.*"
import "android.content.Intent"
import "android.content.Context"
import "java.io.File"
import "android.net.Uri"

local Contacts = {}
local path = this.getLuaExtDir("Plugins").."/Contacts"
local filePath = path .. "/data_contacts"

local dir = File(path)
if not dir.exists() then
    dir.mkdirs()
end

local function readContacts(t, path)
    local file, err = io.open(path, "r")
    if err ~= nil then
        file = io.open(path, "w")
    else
        for line in io.lines(path) do
            local s = string.gsub(line, '\\n', '\n')
            table.insert(t, s)
        end
        file:close()
    end
end

local function writeContacts(t, path)
    local file = io.open(path, 'w')
    for _, v in ipairs(t) do
        file:write(string.gsub(v, '\n', '\\n')..'\n')
    end
    file:close()
end

local function moveItem(t, k, itemKey)
    local s = t[k + 1]
    if itemKey == 0 then
        table.insert(t, 1, s)
        table.remove(t, k + 2)
    elseif itemKey == 1 then
        table.remove(t, k + 1)
    end
end

local editLayout = {
    LinearLayout,
    orientation = "vertical",
    {
        EditText,
        id = "nameField",
        hint = "Enter name"
    },
    {
        EditText,
        id = "numberField",
        hint = "Enter phone number"
    }
}

local edit = loadlayout(editLayout)
local nameField = edit.getChildAt(0)
local numberField = edit.getChildAt(1)

local et = LuaDialog()
et.setTitle("Add Contact")
et.setView(edit)
et.setButton("OK", function()
    local name = nameField.getText().toString()
    local number = numberField.getText().toString()
    if name ~= '' and number ~= '' then
        table.insert(Contacts, name .. " - " .. number)
        nameField.setText('')
        numberField.setText('')
        writeContacts(Contacts, filePath)
    end
    grid.adapter = SingleLineAdapter(service, String(Contacts))
    dlg.show()
end)

-- Help dialog
local readme = LuaDialog(service)
readme.setTitle("Contacts")
readme.setMessage("This is a contacts extension. You can store contacts and manage them. To add a new contact, hit the add button, enter the name and phone number, then press OK.")
readme.setButton("Close", function()
    dlg.show()
end)

-- Long press menu
local menuItem = {
    "Move to top",
    "Delete selected contact",
}
local menuDlg = LuaDialog(service)
menuDlg.setItems(menuItem)

-- Main interface
local layout = {
    GridView,
    id = "grid",
    numColumns = 1,
    layout_width = "fill",
    layout_height = "fill"
}

dlg = LuaDialog(service)
dlg.View = loadlayout(layout)
readContacts(Contacts, filePath)
grid.adapter = SingleLineAdapter(service, String(Contacts))
dlg.setButton("Close")
dlg.setButton2("Add", function(v)
    task(50, function()
        et.show()
    end)
end)
dlg.setButton3("Help", function(v)
    readme.show()
end)

dlg.show()

grid.onItemClick = function(l, v)
    local contact = v.text:match("%- (.+)$")
    if contact then
service.callPhone(contact)
dlg.dismiss()
    end
end

grid.onItemLongClick = function(l, v, p, i)
    dlg.dismiss()
    local vibrator = this.getSystemService(Context.VIBRATOR_SERVICE)
    vibrator.vibrate(long{50, 30}, -1)
    menuDlg.show()
    menuDlg.onItemClick = function(_, _, itemKey)
        menuDlg.dismiss()
        moveItem(Contacts, i, itemKey)
        writeContacts(Contacts, filePath)
        -- Reload contact list
        grid.adapter = SingleLineAdapter(service, String(Contacts))
        dlg.show()
    end
end